##############################################################################
#
# Useful (but simplified) types.
#
# $Id: types.tpl,v 1.4 2002/07/23 09:35:35 cons Exp $
#
##############################################################################

declaration template types;

##############################################################################
#
# simple types
#

# unsigned long
#(old style) define type ulong = long with self >= 0;
define type ulong = long(0..);

# unsigned double
define type udouble = double(0..);

# IPv4 address in dotted number notation
define type ipv4 = string with {
  result = matches(self, '^(\d+)\.(\d+)\.(\d+)\.(\d+)$');
  if (length(result) == 0)
    return("bad string");
  i = 1;
  while (i <= 4) {
    x = to_long(result[i]);
    if (x > 255)
      return("chunk " + to_string(i) + " too big: " + result[i]);
    i = i + 1;
  };
  return(true);
};

##############################################################################
#
# hardware types
#

# memory record
define type memory_t = {
  "size" : ulong
};

# CPU record
define type cpu_t = {
  "vendor" : string
  "model"  : string
  "speed"  : udouble
};

# device record (describing some hardware devices such as disks)
define type device_t = {
  "type"     : string with match(self, '^(disk|cd|net)$')
  "vendor"   : string
  "model"    : string
  "size"     ? ulong
  "driver"   ? string
  "address"  ? string
};

# hardware record (describing some complete hardware information)
define type hardware_t = {
  "vendor"  : string
  "model"   : string
  "serial"  : string
  "memory"  : memory_t
  "cpus"    : cpu_t[1..]   # list of at least one CPU
  "devices" : device_t{}   # table of devices, indexed by names such as hda
};

##############################################################################
#
# system types
#

# mount record (describing what will end up in /etc/fstab)
define type mount_t = {
  "device"  ? string
  "path"    : string
  "type"    : string
  "name"    ? string
  "options" ? string[]
};

# partition record (describing how to partition the disks)
define type partition_t = {
  "disk" : string with value("/hardware/devices/"+self+"/type") == "disk"
  "type" : string
  "size" : ulong
  "id"   : ulong
};

# system record (describing some of the system configuration)
define type system_t = {
  "mounts"     : mount_t[1..]   # list of at least one mount
  "partitions" ? partition_t{}  # table of partitions, indexed by e.g. hda1
};

##############################################################################
#
# root type
#

# type of the root of the configuration information
define type root_t = {
  "hardware" : hardware_t    # hardware subtree
  "system"   : system_t      # system subtree
};

# declare that root is indeed of the root type
type "/" = root_t;
